@@ -2,83 +2,24 @@
 
 from __future__ import print_function
 
+import random
+
 import numpy as np
+import pytest
 
 from pandas.compat import lrange
-from pandas import (DataFrame, Series, MultiIndex, Timestamp,
-                    date_range)
-
-from pandas.util.testing import (assert_series_equal,
-                                 assert_frame_equal,
-                                 assertRaisesRegexp)
-
-import pandas.util.testing as tm
 
+import pandas as pd
+from pandas import (
+    Categorical, DataFrame, IntervalIndex, MultiIndex, NaT, Series, Timestamp,
+    date_range)
+from pandas.api.types import CategoricalDtype
 from pandas.tests.frame.common import TestData
+import pandas.util.testing as tm
+from pandas.util.testing import assert_frame_equal, assert_series_equal
 
 
-class TestDataFrameSorting(tm.TestCase, TestData):
-
-    _multiprocess_can_split_ = True
-
-    def test_sort_index(self):
-        # GH13496
-
-        frame = DataFrame(np.arange(16).reshape(4, 4), index=[1, 2, 3, 4],
-                          columns=['A', 'B', 'C', 'D'])
-
-        # axis=0 : sort rows by index labels
-        unordered = frame.ix[[3, 2, 4, 1]]
-        result = unordered.sort_index(axis=0)
-        expected = frame
-        assert_frame_equal(result, expected)
-
-        result = unordered.sort_index(ascending=False)
-        expected = frame[::-1]
-        assert_frame_equal(result, expected)
-
-        # axis=1 : sort columns by column names
-        unordered = frame.ix[:, [2, 1, 3, 0]]
-        result = unordered.sort_index(axis=1)
-        assert_frame_equal(result, frame)
-
-        result = unordered.sort_index(axis=1, ascending=False)
-        expected = frame.ix[:, ::-1]
-        assert_frame_equal(result, expected)
-
-    def test_sort_index_multiindex(self):
-        # GH13496
-
-        # sort rows by specified level of multi-index
-        mi = MultiIndex.from_tuples([[2, 1, 3], [1, 1, 1]], names=list('ABC'))
-        df = DataFrame([[1, 2], [3, 4]], mi)
-
-        result = df.sort_index(level='A', sort_remaining=False)
-        expected = df.sortlevel('A', sort_remaining=False)
-        assert_frame_equal(result, expected)
-
-        # sort columns by specified level of multi-index
-        df = df.T
-        result = df.sort_index(level='A', axis=1, sort_remaining=False)
-        expected = df.sortlevel('A', axis=1, sort_remaining=False)
-        assert_frame_equal(result, expected)
-
-        # MI sort, but no level: sort_level has no effect
-        mi = MultiIndex.from_tuples([[1, 1, 3], [1, 1, 1]], names=list('ABC'))
-        df = DataFrame([[1, 2], [3, 4]], mi)
-        result = df.sort_index(sort_remaining=False)
-        expected = df.sort_index()
-        assert_frame_equal(result, expected)
-
-    def test_sort(self):
-        frame = DataFrame(np.arange(16).reshape(4, 4), index=[1, 2, 3, 4],
-                          columns=['A', 'B', 'C', 'D'])
-
-        # 9816 deprecated
-        with tm.assert_produces_warning(FutureWarning):
-            frame.sort(columns='A')
-        with tm.assert_produces_warning(FutureWarning):
-            frame.sort()
+class TestDataFrameSorting(TestData):
 
     def test_sort_values(self):
         frame = DataFrame([[1, 1, 2], [3, 1, 0], [4, 5, 6]],
@@ -87,12 +28,12 @@ class TestDataFrameSorting(tm.TestCase, TestData):
         # by column (axis=0)
         sorted_df = frame.sort_values(by='A')
         indexer = frame['A'].argsort().values
-        expected = frame.ix[frame.index[indexer]]
+        expected = frame.loc[frame.index[indexer]]
         assert_frame_equal(sorted_df, expected)
 
         sorted_df = frame.sort_values(by='A', ascending=False)
         indexer = indexer[::-1]
-        expected = frame.ix[frame.index[indexer]]
+        expected = frame.loc[frame.index[indexer]]
         assert_frame_equal(sorted_df, expected)
 
         sorted_df = frame.sort_values(by='A', ascending=False)
@@ -113,7 +54,7 @@ class TestDataFrameSorting(tm.TestCase, TestData):
         sorted_df = frame.sort_values(by=['B', 'A'], ascending=[True, False])
         assert_frame_equal(sorted_df, expected)
 
-        self.assertRaises(ValueError, lambda: frame.sort_values(
+        pytest.raises(ValueError, lambda: frame.sort_values(
             by=['A', 'B'], axis=2, inplace=True))
 
         # by row (axis=1): GH 10806
@@ -138,7 +79,7 @@ class TestDataFrameSorting(tm.TestCase, TestData):
         assert_frame_equal(sorted_df, expected)
 
         msg = r'Length of ascending \(5\) != length of by \(2\)'
-        with assertRaisesRegexp(ValueError, msg):
+        with pytest.raises(ValueError, match=msg):
             frame.sort_values(by=['A', 'B'], axis=0, ascending=[True] * 5)
 
     def test_sort_values_inplace(self):
@@ -165,21 +106,6 @@ class TestDataFrameSorting(tm.TestCase, TestData):
         expected = frame.sort_values(by=['A', 'B'], ascending=False)
         assert_frame_equal(sorted_df, expected)
 
-    def test_sort_index_categorical_index(self):
-
-        df = (DataFrame({'A': np.arange(6, dtype='int64'),
-                         'B': Series(list('aabbca'))
-                         .astype('category', categories=list('cab'))})
-              .set_index('B'))
-
-        result = df.sort_index()
-        expected = df.iloc[[4, 0, 1, 5, 2, 3]]
-        assert_frame_equal(result, expected)
-
-        result = df.sort_index(ascending=False)
-        expected = df.iloc[[3, 2, 5, 1, 0, 4]]
-        assert_frame_equal(result, expected)
-
     def test_sort_nan(self):
         # GH3917
         nan = np.nan
@@ -227,7 +153,7 @@ class TestDataFrameSorting(tm.TestCase, TestData):
              'B': [5, 9, 2, nan, 5, 5, 4]},
             index=[2, 0, 3, 1, 6, 4, 5])
         sorted_df = df.sort_values(['A', 'B'], ascending=[
-                                   1, 0], na_position='first')
+            1, 0], na_position='first')
         assert_frame_equal(sorted_df, expected)
 
         # na_position='last', not order
@@ -236,7 +162,7 @@ class TestDataFrameSorting(tm.TestCase, TestData):
              'B': [4, 5, 5, nan, 2, 9, 5]},
             index=[5, 4, 6, 1, 3, 0, 2])
         sorted_df = df.sort_values(['A', 'B'], ascending=[
-                                   0, 1], na_position='last')
+            0, 1], na_position='last')
         assert_frame_equal(sorted_df, expected)
 
         # Test DataFrame with nan label
@@ -305,8 +231,122 @@ class TestDataFrameSorting(tm.TestCase, TestData):
                                    kind='mergesort')
         assert_frame_equal(sorted_df, expected)
 
+    def test_stable_categorial(self):
+        # GH 16793
+        df = DataFrame({
+            'x': pd.Categorical(np.repeat([1, 2, 3, 4], 5), ordered=True)
+        })
+        expected = df.copy()
+        sorted_df = df.sort_values('x', kind='mergesort')
+        assert_frame_equal(sorted_df, expected)
+
+    def test_sort_datetimes(self):
+
+        # GH 3461, argsort / lexsort differences for a datetime column
+        df = DataFrame(['a', 'a', 'a', 'b', 'c', 'd', 'e', 'f', 'g'],
+                       columns=['A'],
+                       index=date_range('20130101', periods=9))
+        dts = [Timestamp(x)
+               for x in ['2004-02-11', '2004-01-21', '2004-01-26',
+                         '2005-09-20', '2010-10-04', '2009-05-12',
+                         '2008-11-12', '2010-09-28', '2010-09-28']]
+        df['B'] = dts[::2] + dts[1::2]
+        df['C'] = 2.
+        df['A1'] = 3.
+
+        df1 = df.sort_values(by='A')
+        df2 = df.sort_values(by=['A'])
+        assert_frame_equal(df1, df2)
+
+        df1 = df.sort_values(by='B')
+        df2 = df.sort_values(by=['B'])
+        assert_frame_equal(df1, df2)
+
+        df1 = df.sort_values(by='B')
+
+        df2 = df.sort_values(by=['C', 'B'])
+        assert_frame_equal(df1, df2)
+
+    def test_frame_column_inplace_sort_exception(self):
+        s = self.frame['A']
+        with pytest.raises(ValueError, match="This Series is a view"):
+            s.sort_values(inplace=True)
+
+        cp = s.copy()
+        cp.sort_values()  # it works!
+
+    def test_sort_nat_values_in_int_column(self):
+
+        # GH 14922: "sorting with large float and multiple columns incorrect"
+
+        # cause was that the int64 value NaT was considered as "na". Which is
+        # only correct for datetime64 columns.
+
+        int_values = (2, int(NaT))
+        float_values = (2.0, -1.797693e308)
+
+        df = DataFrame(dict(int=int_values, float=float_values),
+                       columns=["int", "float"])
+
+        df_reversed = DataFrame(dict(int=int_values[::-1],
+                                     float=float_values[::-1]),
+                                columns=["int", "float"],
+                                index=[1, 0])
+
+        # NaT is not a "na" for int64 columns, so na_position must not
+        # influence the result:
+        df_sorted = df.sort_values(["int", "float"], na_position="last")
+        assert_frame_equal(df_sorted, df_reversed)
+
+        df_sorted = df.sort_values(["int", "float"], na_position="first")
+        assert_frame_equal(df_sorted, df_reversed)
+
+        # reverse sorting order
+        df_sorted = df.sort_values(["int", "float"], ascending=False)
+        assert_frame_equal(df_sorted, df)
+
+        # and now check if NaT is still considered as "na" for datetime64
+        # columns:
+        df = DataFrame(dict(datetime=[Timestamp("2016-01-01"), NaT],
+                            float=float_values), columns=["datetime", "float"])
+
+        df_reversed = DataFrame(dict(datetime=[NaT, Timestamp("2016-01-01")],
+                                     float=float_values[::-1]),
+                                columns=["datetime", "float"],
+                                index=[1, 0])
+
+        df_sorted = df.sort_values(["datetime", "float"], na_position="first")
+        assert_frame_equal(df_sorted, df_reversed)
+
+        df_sorted = df.sort_values(["datetime", "float"], na_position="last")
+        assert_frame_equal(df_sorted, df)
+
+        # Ascending should not affect the results.
+        df_sorted = df.sort_values(["datetime", "float"], ascending=False)
+        assert_frame_equal(df_sorted, df)
+
+    def test_sort_nat(self):
+
+        # GH 16836
+
+        d1 = [Timestamp(x) for x in ['2016-01-01', '2015-01-01',
+                                     np.nan, '2016-01-01']]
+        d2 = [Timestamp(x) for x in ['2017-01-01', '2014-01-01',
+                                     '2016-01-01', '2015-01-01']]
+        df = pd.DataFrame({'a': d1, 'b': d2}, index=[0, 1, 2, 3])
+
+        d3 = [Timestamp(x) for x in ['2015-01-01', '2016-01-01',
+                                     '2016-01-01', np.nan]]
+        d4 = [Timestamp(x) for x in ['2014-01-01', '2015-01-01',
+                                     '2017-01-01', '2016-01-01']]
+        expected = pd.DataFrame({'a': d3, 'b': d4}, index=[1, 3, 0, 2])
+        sorted_df = df.sort_values(by=['a', 'b'], )
+        tm.assert_frame_equal(sorted_df, expected)
+
+
+class TestDataFrameSortIndexKinds(TestData):
+
     def test_sort_index_multicolumn(self):
-        import random
         A = np.arange(5).repeat(20)
         B = np.tile(np.arange(5), 20)
         random.shuffle(A)
@@ -344,13 +384,13 @@ class TestDataFrameSorting(tm.TestCase, TestData):
                           columns=['A', 'B', 'C', 'D'])
 
         # axis=0
-        unordered = frame.ix[[3, 2, 4, 1]]
+        unordered = frame.loc[[3, 2, 4, 1]]
         a_id = id(unordered['A'])
         df = unordered.copy()
         df.sort_index(inplace=True)
         expected = frame
         assert_frame_equal(df, expected)
-        self.assertNotEqual(a_id, id(df['A']))
+        assert a_id != id(df['A'])
 
         df = unordered.copy()
         df.sort_index(ascending=False, inplace=True)
@@ -358,7 +398,7 @@ class TestDataFrameSorting(tm.TestCase, TestData):
         assert_frame_equal(df, expected)
 
         # axis=1
-        unordered = frame.ix[:, ['D', 'B', 'C', 'A']]
+        unordered = frame.loc[:, ['D', 'B', 'C', 'A']]
         df = unordered.copy()
         df.sort_index(axis=1, inplace=True)
         expected = frame
@@ -366,7 +406,7 @@ class TestDataFrameSorting(tm.TestCase, TestData):
 
         df = unordered.copy()
         df.sort_index(axis=1, ascending=False, inplace=True)
-        expected = frame.ix[:, ::-1]
+        expected = frame.iloc[:, ::-1]
         assert_frame_equal(df, expected)
 
     def test_sort_index_different_sortorder(self):
@@ -407,26 +447,26 @@ class TestDataFrameSorting(tm.TestCase, TestData):
         df = DataFrame([lrange(5, 9), lrange(4)],
                        columns=['a', 'a', 'b', 'b'])
 
-        with assertRaisesRegexp(ValueError, 'duplicate'):
+        with pytest.raises(ValueError, match='not unique'):
             # use .sort_values #9816
             with tm.assert_produces_warning(FutureWarning):
                 df.sort_index(by='a')
-        with assertRaisesRegexp(ValueError, 'duplicate'):
+        with pytest.raises(ValueError, match='not unique'):
             df.sort_values(by='a')
 
-        with assertRaisesRegexp(ValueError, 'duplicate'):
+        with pytest.raises(ValueError, match='not unique'):
             # use .sort_values #9816
             with tm.assert_produces_warning(FutureWarning):
                 df.sort_index(by=['a'])
-        with assertRaisesRegexp(ValueError, 'duplicate'):
+        with pytest.raises(ValueError, match='not unique'):
             df.sort_values(by=['a'])
 
-        with assertRaisesRegexp(ValueError, 'duplicate'):
+        with pytest.raises(ValueError, match='not unique'):
             # use .sort_values #9816
             with tm.assert_produces_warning(FutureWarning):
                 # multi-column 'by' is separate codepath
                 df.sort_index(by=['a', 'b'])
-        with assertRaisesRegexp(ValueError, 'duplicate'):
+        with pytest.raises(ValueError, match='not unique'):
             # multi-column 'by' is separate codepath
             df.sort_values(by=['a', 'b'])
 
@@ -434,11 +474,11 @@ class TestDataFrameSorting(tm.TestCase, TestData):
         # GH4370
         df = DataFrame(np.random.randn(4, 2),
                        columns=MultiIndex.from_tuples([('a', 0), ('a', 1)]))
-        with assertRaisesRegexp(ValueError, 'levels'):
+        with pytest.raises(ValueError, match='level'):
             # use .sort_values #9816
             with tm.assert_produces_warning(FutureWarning):
                 df.sort_index(by='a')
-        with assertRaisesRegexp(ValueError, 'levels'):
+        with pytest.raises(ValueError, match='level'):
             df.sort_values(by='a')
 
         # convert tuples to a list of tuples
@@ -453,41 +493,178 @@ class TestDataFrameSorting(tm.TestCase, TestData):
         result = df.sort_values(by=('a', 1))
         assert_frame_equal(result, expected)
 
-    def test_sortlevel(self):
+    def test_sort_index_level(self):
         mi = MultiIndex.from_tuples([[1, 1, 3], [1, 1, 1]], names=list('ABC'))
         df = DataFrame([[1, 2], [3, 4]], mi)
-        res = df.sortlevel('A', sort_remaining=False)
+        res = df.sort_index(level='A', sort_remaining=False)
         assert_frame_equal(df, res)
 
-        res = df.sortlevel(['A', 'B'], sort_remaining=False)
+        res = df.sort_index(level=['A', 'B'], sort_remaining=False)
         assert_frame_equal(df, res)
 
-    def test_sort_datetimes(self):
+    def test_sort_index_categorical_index(self):
 
-        # GH 3461, argsort / lexsort differences for a datetime column
-        df = DataFrame(['a', 'a', 'a', 'b', 'c', 'd', 'e', 'f', 'g'],
-                       columns=['A'],
-                       index=date_range('20130101', periods=9))
-        dts = [Timestamp(x)
-               for x in ['2004-02-11', '2004-01-21', '2004-01-26',
-                         '2005-09-20', '2010-10-04', '2009-05-12',
-                         '2008-11-12', '2010-09-28', '2010-09-28']]
-        df['B'] = dts[::2] + dts[1::2]
-        df['C'] = 2.
-        df['A1'] = 3.
+        df = (DataFrame({'A': np.arange(6, dtype='int64'),
+                         'B': Series(list('aabbca'))
+                        .astype(CategoricalDtype(list('cab')))})
+              .set_index('B'))
 
-        df1 = df.sort_values(by='A')
-        df2 = df.sort_values(by=['A'])
-        assert_frame_equal(df1, df2)
+        result = df.sort_index()
+        expected = df.iloc[[4, 0, 1, 5, 2, 3]]
+        assert_frame_equal(result, expected)
 
-        df1 = df.sort_values(by='B')
-        df2 = df.sort_values(by=['B'])
-        assert_frame_equal(df1, df2)
+        result = df.sort_index(ascending=False)
+        expected = df.iloc[[3, 2, 5, 1, 0, 4]]
+        assert_frame_equal(result, expected)
 
-    def test_frame_column_inplace_sort_exception(self):
-        s = self.frame['A']
-        with assertRaisesRegexp(ValueError, "This Series is a view"):
-            s.sort_values(inplace=True)
+    def test_sort_index(self):
+        # GH13496
 
-        cp = s.copy()
-        cp.sort_values()  # it works!
+        frame = DataFrame(np.arange(16).reshape(4, 4), index=[1, 2, 3, 4],
+                          columns=['A', 'B', 'C', 'D'])
+
+        # axis=0 : sort rows by index labels
+        unordered = frame.loc[[3, 2, 4, 1]]
+        result = unordered.sort_index(axis=0)
+        expected = frame
+        assert_frame_equal(result, expected)
+
+        result = unordered.sort_index(ascending=False)
+        expected = frame[::-1]
+        assert_frame_equal(result, expected)
+
+        # axis=1 : sort columns by column names
+        unordered = frame.iloc[:, [2, 1, 3, 0]]
+        result = unordered.sort_index(axis=1)
+        assert_frame_equal(result, frame)
+
+        result = unordered.sort_index(axis=1, ascending=False)
+        expected = frame.iloc[:, ::-1]
+        assert_frame_equal(result, expected)
+
+    @pytest.mark.parametrize("level", ['A', 0])  # GH 21052
+    def test_sort_index_multiindex(self, level):
+        # GH13496
+
+        # sort rows by specified level of multi-index
+        mi = MultiIndex.from_tuples([
+            [2, 1, 3], [2, 1, 2], [1, 1, 1]], names=list('ABC'))
+        df = DataFrame([[1, 2], [3, 4], [5, 6]], index=mi)
+
+        expected_mi = MultiIndex.from_tuples([
+            [1, 1, 1],
+            [2, 1, 2],
+            [2, 1, 3]], names=list('ABC'))
+        expected = pd.DataFrame([
+            [5, 6],
+            [3, 4],
+            [1, 2]], index=expected_mi)
+        result = df.sort_index(level=level)
+        assert_frame_equal(result, expected)
+
+        # sort_remaining=False
+        expected_mi = MultiIndex.from_tuples([
+            [1, 1, 1],
+            [2, 1, 3],
+            [2, 1, 2]], names=list('ABC'))
+        expected = pd.DataFrame([
+            [5, 6],
+            [1, 2],
+            [3, 4]], index=expected_mi)
+        result = df.sort_index(level=level, sort_remaining=False)
+        assert_frame_equal(result, expected)
+
+    def test_sort_index_intervalindex(self):
+        # this is a de-facto sort via unstack
+        # confirming that we sort in the order of the bins
+        y = Series(np.random.randn(100))
+        x1 = Series(np.sign(np.random.randn(100)))
+        x2 = pd.cut(Series(np.random.randn(100)),
+                    bins=[-3, -0.5, 0, 0.5, 3])
+        model = pd.concat([y, x1, x2], axis=1, keys=['Y', 'X1', 'X2'])
+
+        result = model.groupby(['X1', 'X2'], observed=True).mean().unstack()
+        expected = IntervalIndex.from_tuples(
+            [(-3.0, -0.5), (-0.5, 0.0),
+             (0.0, 0.5), (0.5, 3.0)],
+            closed='right')
+        result = result.columns.levels[1].categories
+        tm.assert_index_equal(result, expected)
+
+    def test_sort_index_na_position_with_categories(self):
+        # GH 22556
+        # Positioning missing value properly when column is Categorical.
+        categories = ['A', 'B', 'C']
+        category_indices = [0, 2, 4]
+        list_of_nans = [np.nan, np.nan]
+        na_indices = [1, 3]
+        na_position_first = 'first'
+        na_position_last = 'last'
+        column_name = 'c'
+
+        reversed_categories = sorted(categories, reverse=True)
+        reversed_category_indices = sorted(category_indices, reverse=True)
+        reversed_na_indices = sorted(na_indices, reverse=True)
+
+        df = pd.DataFrame({
+            column_name: pd.Categorical(['A', np.nan, 'B', np.nan, 'C'],
+                                        categories=categories,
+                                        ordered=True)})
+        # sort ascending with na first
+        result = df.sort_values(by=column_name,
+                                ascending=True,
+                                na_position=na_position_first)
+        expected = DataFrame({
+            column_name: Categorical(list_of_nans + categories,
+                                     categories=categories,
+                                     ordered=True)
+        }, index=na_indices + category_indices)
+
+        assert_frame_equal(result, expected)
+
+        # sort ascending with na last
+        result = df.sort_values(by=column_name,
+                                ascending=True,
+                                na_position=na_position_last)
+        expected = DataFrame({
+            column_name: Categorical(categories + list_of_nans,
+                                     categories=categories,
+                                     ordered=True)
+        }, index=category_indices + na_indices)
+
+        assert_frame_equal(result, expected)
+
+        # sort descending with na first
+        result = df.sort_values(by=column_name,
+                                ascending=False,
+                                na_position=na_position_first)
+        expected = DataFrame({
+            column_name: Categorical(list_of_nans + reversed_categories,
+                                     categories=categories,
+                                     ordered=True)
+        }, index=reversed_na_indices + reversed_category_indices)
+
+        assert_frame_equal(result, expected)
+
+        # sort descending with na last
+        result = df.sort_values(by=column_name,
+                                ascending=False,
+                                na_position=na_position_last)
+        expected = DataFrame({
+            column_name: Categorical(reversed_categories + list_of_nans,
+                                     categories=categories,
+                                     ordered=True)
+        }, index=reversed_category_indices + reversed_na_indices)
+
+        assert_frame_equal(result, expected)
+
+    def test_sort_index_na_position_with_categories_raises(self):
+        df = pd.DataFrame({
+            'c': pd.Categorical(['A', np.nan, 'B', np.nan, 'C'],
+                                categories=['A', 'B', 'C'],
+                                ordered=True)})
+
+        with pytest.raises(ValueError):
+            df.sort_values(by='c',
+                           ascending=False,
+                           na_position='bad_position')
